package reproduction;

import java.sql.*;

public class Main {

  private static final String SETUP_DDL =
    "drop table if EXISTS example1;\n" +
    "drop table if EXISTS example2;\n" +
    "drop table if EXISTS example3;\n" +
    "\n" +
    "create table example1 (\n" +
    "  ID int primary key IDENTITY(1,1) NOT NULL,\n" +
    "  NAME varchar(255) NOT NULL\n" +
    ");\n" +
    "create table example2 (\n" +
    "  ID int primary key IDENTITY(1,1) NOT NULL,\n" +
    "  NAME varchar(255) NOT NULL\n" +
    ");\n" +
    "create table example3 (\n" +
    "  ID int primary key IDENTITY(1,1) NOT NULL,\n" +
    "  NAME varchar(255) NOT NULL\n" +
    ");";

  private static final String TEAR_DOWN_DDL =
    "drop table if EXISTS example1;\n" +
    "drop table if EXISTS example2;\n" +
    "drop table if EXISTS example3;";

  // Insane query, just to assure it takes long.
  private static final String QUERY =
    "select e1.*\n" +
      "from example1 e1\n" +
      ",    example2 e2\n" +
      ",    example3 e3\n" +
      ",    example1 e4\n" +
      "where e1.name = 'abc'\n" +
      "or    e2.name = 'def'\n" +
      "or    e3.name = 'ghi'\n" +
      "or    e4.name = 'xxx'\n" +
      "and e1.name not in \n" +
      "  (select name\n" +
      "   from example2\n" +
      "  )\n" +
      "and e2.name not in \n" +
      "  (select name\n" +
      "   from example1\n" +
      "  )\n" +
      "and e3.name not in \n" +
      "  (select name\n" +
      "   from example2\n" +
      "  )\n" +
      "and e4.name not in \n" +
      "  (select name\n" +
      "   from example3\n" +
      "  );";

  public static void main(String[] args) throws ClassNotFoundException, SQLException {
    Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
    try {
      setup();
      doTest();
    }
    finally {
      tearDown();
    }
  }

  private static void setup() throws SQLException {
    System.out.println("Setup...");

    final String insertTable1 = "insert into example1 values (?)";
    final String insertTable2 = "insert into example2 values (?)";
    final String insertTable3 = "insert into example3 values (?)";
    try (final Connection conn = getConnection()) {
      // Create tables
      try (PreparedStatement st = conn.prepareStatement(SETUP_DDL))
      {
        st.executeUpdate();
      }

      // Add 5000 records to each table
      try (PreparedStatement stTable1 = conn.prepareStatement(insertTable1);
           PreparedStatement stTable2 = conn.prepareStatement(insertTable2);
           PreparedStatement stTable3 = conn.prepareStatement(insertTable3))
      {
        for (int i = 0; i < 10000; i++)
        {
          stTable1.setString(1, "value" + i);
          stTable2.setString(1, "value" + i);
          stTable3.setString(1, "value" + i);

          stTable1.addBatch();
          stTable2.addBatch();
          stTable3.addBatch();
        }

        stTable1.executeBatch();
        stTable2.executeBatch();
        stTable3.executeBatch();
      }
      conn.commit();
    }

    System.out.println("Setup complete.\n");
  }

  private static void tearDown() throws SQLException {
    System.out.println("Teardown started...");
    try (final Connection conn = getConnection();
         final PreparedStatement st = conn.prepareStatement(TEAR_DOWN_DDL)) {

      st.executeUpdate();
      conn.commit();
    }
    System.out.println("Teardown complete.\n");
  }

  private static void doTest() throws SQLException {
    System.out.println("Test started...");
    try (final Connection conn = getConnection();
         final PreparedStatement st = conn.prepareStatement(QUERY);
         final ResultSet rs = st.executeQuery())
    {
      /*
       * It will not get here unless we kill the session in SQLServer using: KILL sessionid. Please do that!
       *
       * However it should *not* get here at all - but it does. Expected is that st.executeQuery() above
       * throws some kind of SQLException with a high error code, as killing the connection in use
       * should be fatal. This should apply to any of the query methods.
       */
      System.out.println("Next: " + rs.next() + ", interrupted=" + Thread.currentThread().isInterrupted());
    }
    System.out.println("Test complete (should not be seen if a kill was issued, exception was expected)\n");
  }

  private static Connection getConnection() throws SQLException {
    final String url = "jdbc:sqlserver://localhost:1433;databaseName=master";
    final Connection conn = DriverManager.getConnection(url, "sa", "TestTest1");
    conn.setAutoCommit(false);
    return conn;
  }
}
